package com.caimei365.manager.service.sys.impl;

import com.caimei365.manager.dao.SystemDao;
import com.caimei365.manager.entity.PaginationVo;
import com.caimei365.manager.entity.ResponseJson;
import com.caimei365.manager.entity.sys.SysMenu;
import com.caimei365.manager.service.sys.SysMenuService;
import com.github.pagehelper.PageHelper;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;
import org.springframework.util.StringUtils;

import javax.annotation.Resource;
import java.util.List;

/**
 * Description
 *
 * @author : Charles
 * @date : 2022/1/6
 */
@Slf4j
@Service
public class SysMenuServiceImpl implements SysMenuService {

    @Resource
    private SystemDao systemDao;

    /**
     * 获取菜单列表
     *
     * @param parentId 父级菜单Id
     * @param status   状态：0启用，1停用
     * @param pageNum  页码
     * @param pageSize 每页大小
     */
    @Override
    public ResponseJson<PaginationVo<SysMenu>> getMenuList(Integer parentId, Integer status, int pageNum, int pageSize) {
        PageHelper.startPage(pageNum, pageSize);
        List<SysMenu> menus = systemDao.getMenuList(parentId, status);
        PaginationVo<SysMenu> pageData = new PaginationVo<>(menus);
        return ResponseJson.success(pageData);
    }

    /**
     * 获取所有启用的菜单列表（树状结构返回）
     */
    @Override
    public ResponseJson<List<SysMenu>> getMenuTree() {
        List<SysMenu> menus = systemDao.getMenuList(0, 0);
        // 递归设置子菜单
        setSubMenusData(menus);
        return ResponseJson.success(menus);
    }

    /**
     * 递归设置子菜单
     */
    private void setSubMenusData(List<SysMenu> menus) {
        if (!CollectionUtils.isEmpty(menus)) {
            for (SysMenu menu : menus) {
                List<SysMenu> submenus = systemDao.getMenuList(menu.getId(), 0);
                if (!CollectionUtils.isEmpty(submenus)) {
                    setSubMenusData(submenus);
                    menu.setSubMenus(submenus);
                }
            }
        }
    }

    /**
     * 根据ID获取菜单
     */
    @Override
    public ResponseJson<SysMenu> getMenu(Integer id) {
        SysMenu menu = systemDao.getMenu(id);
        return ResponseJson.success(menu);
    }

    /**
     * 根据ID更新菜单
     */
    @Override
    public ResponseJson<Void> updateMenu(Integer id, SysMenu menu) {
        menu.setId(id);
        menu.setDelFlag(0);
        systemDao.updateMenu(menu);
        return ResponseJson.success();
    }

    /**
     * 根据ID(可选择的)更新菜单字段
     */
    @Override
    public ResponseJson<Void> updateMenuSelective(Integer id, SysMenu menu) {
        SysMenu dbMenu = systemDao.getMenu(id);
        if (StringUtils.hasLength(menu.getTitle())){
            dbMenu.setTitle(menu.getTitle());
        }
        if (StringUtils.hasLength(menu.getName())){
            dbMenu.setName(menu.getName());
        }
        if (StringUtils.hasLength(menu.getIcon())){
            dbMenu.setIcon(menu.getIcon());
        }
        if (null != menu.getParentId()){
            dbMenu.setParentId(menu.getParentId());
        }
        if (null != menu.getSort()){
            dbMenu.setSort(menu.getSort());
        }
        if (null != menu.getHidden()){
            dbMenu.setHidden(menu.getHidden());
        }
        if (null != menu.getStatus()){
            dbMenu.setStatus(menu.getStatus());
        }
        systemDao.updateMenu(dbMenu);
        return ResponseJson.success();
    }

    /**
     * 添加菜单
     */
    @Override
    public ResponseJson<Void> addMenu(SysMenu menu) {
        if (null == menu.getParentId()) {
            menu.setParentId(0);
        }
        if (!StringUtils.hasLength(menu.getName())) {
            return ResponseJson.error("菜单路由名不能为空！", null);
        }
        if (!StringUtils.hasLength(menu.getTitle())) {
            return ResponseJson.error("菜单名称不能为空！", null);
        }
        // 根据路由名称查询菜单
        SysMenu dbMenu = systemDao.getMenuByName(menu.getName());
        if (null == dbMenu) {
            systemDao.insertMenu(menu);
        } else if (!Integer.valueOf(0).equals(dbMenu.getDelFlag())){
            // 更新已删除菜单
            return updateMenu(dbMenu.getId(), menu);
        } else {
            return ResponseJson.error("菜单路由名已存在", null);
        }
        return ResponseJson.success();
    }

    /**
     * 根据ID删除菜单
     */
    @Override
    public ResponseJson<Void> deleteMenu(Integer id) {
        systemDao.deleteMenu(id);
        return ResponseJson.success();
    }
}
